import { createRoot } from "react-dom/client";
import { Suspense, lazy, StrictMode } from "react";
import "./tailwind.css";
import "./styles/tour-guide.css"; // Import tour guide animations and styles
import "./lib/env-config"; // Import environment config to set up API keys

// Performance metrics logging - helps monitor real user metrics
const reportWebVitals = ({ name, value, delta }: { name: string, value: number, delta: number }) => {
  // Skip development logging to reduce console noise
  if (process.env.NODE_ENV !== 'production') return;
  
  console.log(`[Metrics] ${name}: ${Math.round(value)}ms (Δ${Math.round(delta)}ms)`);
  
  // Send to analytics in production (if available)
  // NOTE: This is just a placeholder for future analytics integration
  const windowWithGtag = window as Window & { gtag?: (...args: any[]) => void };
  if (typeof windowWithGtag.gtag === 'function') {
    windowWithGtag.gtag('event', name, {
      value: Math.round(value),
      metric_id: name,
      metric_value: value,
      metric_delta: delta,
    });
  }
};

// Use the browser's native performance observation API
// This minimizes the need for additional JavaScript before hydration
const setupPerformanceObserver = () => {
  if ('PerformanceObserver' in window) {
    try {
      // CLS observer
      new PerformanceObserver((entryList) => {
        for (const entry of entryList.getEntries()) {
          const metric = entry as PerformanceEntry & { value?: number };
          if (metric.name === 'layout-shift' && metric.value) {
            const cls = {
              name: 'CLS',
              value: metric.value * 1000,
              delta: metric.value * 1000,
            };
            reportWebVitals(cls);
          }
        }
      }).observe({ type: 'layout-shift', buffered: true });
      
      // LCP observer
      new PerformanceObserver((entryList) => {
        for (const entry of entryList.getEntries()) {
          if (entry.name === 'largest-contentful-paint') {
            const metric = {
              name: 'LCP',
              value: entry.startTime,
              delta: entry.startTime,
            };
            reportWebVitals(metric);
          }
        }
      }).observe({ type: 'largest-contentful-paint', buffered: true });
      
      // FID observer
      new PerformanceObserver((entryList) => {
        for (const entry of entryList.getEntries()) {
          const metric = entry as PerformanceEventTiming;
          if (metric.name === 'first-input') {
            const fid = {
              name: 'FID',
              value: metric.processingStart - metric.startTime,
              delta: metric.processingStart - metric.startTime,
            };
            reportWebVitals(fid);
          }
        }
      }).observe({ type: 'first-input', buffered: true });
    } catch (e) {
      console.error('Performance observer setup failed:', e);
    }
  }
};

// Asynchronously load translations in the background (non-blocking)
// This prevents a render-blocking waterfall and initializes i18n exactly once
const preloadTranslations = () => {
  console.log('[main] Preloading i18n configuration...');
  
  // Use dynamic import with the ensureI18nInitialized function
  return import("./i18n/config")
    .then(i18nModule => {
      // Call the exported function to ensure initialization
      return i18nModule.ensureI18nInitialized().then(() => {
        console.log('[main] i18n configuration loaded successfully');
        return i18nModule.default; // Return the i18n instance
      });
    })
    .catch(err => {
      console.error('[main] Failed to preload i18n:', err);
      // In case of error, try to clear localStorage flag to allow a clean retry
      try {
        localStorage.removeItem('__PitStop360_i18n_initialized__');
      } catch (e) {
        // Ignore localStorage errors
      }
    });
};

// Begin preloading translations immediately - parallel loading
// This is the ONE and ONLY place where i18n should be initialized
preloadTranslations();

// Optimize App chunk loading with priority hints
const App = lazy(() => 
  import(/* webpackPrefetch: true */ "./App").catch(error => {
    console.error("Error loading App:", error);
    return Promise.reject(error);
  })
);

// Import our enhanced animated loader (keep in main chunk for fast initial render)
import { AnimatedLoader } from "./components/ui/animated-loader";

// Professional animated loading component
const AppLoading = () => <AnimatedLoader />;

// Get DOM reference only once
const rootElement = document.getElementById("root");

// Immediately start hydration - fast path
if (rootElement) {
  createRoot(rootElement).render(
    <StrictMode>
      <Suspense fallback={<AppLoading />}>
        <App />
      </Suspense>
    </StrictMode>
  );
}

// Register service worker after initial render
// Use a simpler implementation to avoid any React hook issues
const registerServiceWorker = () => {
  // Make sure service worker is supported in this browser
  if (!('serviceWorker' in navigator)) {
    console.log('Service Worker API not supported in this browser');
    return;
  }

  // Register the service worker
  window.addEventListener('load', () => {
    try {
      navigator.serviceWorker.register('/service-worker.js')
        .then(registration => {
          console.log('Service Worker registered with scope:', registration.scope);
          
          // Set up a basic message handler
          navigator.serviceWorker.addEventListener('message', (event) => {
            const data = event.data;
            
            // Log any update notifications
            if (data && data.type === 'SERVICE_WORKER_UPDATED') {
              console.log(`[ServiceWorker] New service worker activated: ${data.cacheVersion}`);
            }
          });
        })
        .catch(error => {
          console.error('Service Worker registration failed:', error);
        });
    } catch (error) {
      console.error('Error registering service worker:', error);
    }
  });
};

// Set up performance monitoring
setupPerformanceObserver();

// Register the service worker
registerServiceWorker();
